/* 
 * FileAndStringTransferHandler.java - TransferHandler fr JTextArea-Komponenten, der Dateien und Strings untersttzt
 *
 */
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/*
 * Transfer-Handler fr JTextArea-Komponenten,
 * der Dateien und Strings verarbeitet
 */
class FileAndStringTransferHandler extends TransferHandler {

    Position p0 = null, p1 = null;  // Anfang und Ende zu exportierender
                                    // Drag-Strings
 
    private JTextArea source;       // Drag-Quelle, wird bei String-Drag 
                                    // (Methode createTransferable())
                                    // gespeichert, um bei Drop feststellen
                                    // zu knnen, ob Quelle und Ziel
                                    // identisch sind. Wenn dann auch die
                                    // Drop-Position im Bereich der 
                                    // Drag-Auswahl liegt, muss nichts
                                    // verndert werden.

    private boolean shouldRemove;   // Wird auf false gesetzt, wenn Drop- 
                                    // Position im Drag-Bereich liegt (s.o.),
                                    // um zu verhindern, dass exportDone()
                                    // die Drag-Daten lscht

    FileAndStringTransferHandler() {
    }


    /*
     * Prft, ob die angebotenen Daten in einem DataFlavor 
     * angeboten werden, der fr die Target-Komponente geeignet ist.
     */
    public boolean canImport(TransferHandler.TransferSupport support) {

        if ( !(support.isDataFlavorSupported(DataFlavor.stringFlavor)) &&
             !(support.isDataFlavorSupported(DataFlavor.javaFileListFlavor)))
            return false;
            
        return true;
    }


    /*
     * Hilfsmethoden, die feststellen, ob eine bestimmte Art von Daten vorliegt.
     */    
    private boolean hasFileFlavor(DataFlavor[] flavors) {
        for (int i = 0; i < flavors.length; i++) {
            if (flavors[i].equals(DataFlavor.javaFileListFlavor)) 
                return true;
        }
        return false;
    }

    private boolean hasStringFlavor(DataFlavor[] flavors) {
        for (int i = 0; i < flavors.length; i++) {
            if (flavors[i].equals(DataFlavor.stringFlavor)) 
                return true;
        }
        return false;
    }


   /**
     * versucht die Daten aus dem Transferable-Objekt t in der Komponente
     *abzulegen
     */
    public boolean importData(TransferHandler.TransferSupport support) {
    
        // Vorab prfen, ob die angebotenen Daten in 
        // einem Data Flavor angeboten werden, der fr die Target-Komponente
        // geeignet ist.
        // muss wegen Paste-Operation erneut berprft werden
        if (!canImport(support)) 
            return false;

        if (!(support.getComponent() instanceof JTextArea))
            return false;
            
        JTextArea target = (JTextArea) support.getComponent();
        Transferable t = support.getTransferable();        
                

        try {
            if (hasFileFlavor(t.getTransferDataFlavors())) {
                // Transferable-Objekt enthlt eine (oder mehrere) Dateien.
                // Lese die erste Datei und fge ihren Inhalt in die 
                // Target-Komponente ein

                FileReader in = null;
                StringBuilder str = new StringBuilder();
                java.util.List files = (java.util.List) 
                             t.getTransferData(DataFlavor.javaFileListFlavor);

                if (files.size() > 0) {

                    // nur erste Datei lesen
                    File f = (File) files.get(0);

                    try {
                        in = new FileReader(f);

                        // Dateiinhalt in String lesen
                        int countBytes = 0;
                        char[] bytesRead = new char[512];
       
                        while( (countBytes = in.read(bytesRead)) > 0) 
                           str.append(bytesRead, 0, countBytes);   

                        // Drop-Position bestimmen
                        JTextComponent.DropLocation dropPos = 
                             (JTextComponent.DropLocation) support.getDropLocation();
                        int pos = dropPos.getIndex();
                                target.insert(str.toString(), pos);
                        
                    } catch (IOException e) {
                        System.err.println(f + 
                                           " kann nicht eingelesen werden");
                    } finally {
                        if (in != null) 
                           in.close();
                    }

                return true;
                }

            } else if (hasStringFlavor(t.getTransferDataFlavors())) {
                String str = 
                          (String) t.getTransferData(DataFlavor.stringFlavor);
                 
                if (support.isDrop()) {
                   // Fr Drop-Aktionen Drop-Position bestimmen und String einfgen                  
                   JTextComponent.DropLocation dropStelle = 
                           (JTextComponent.DropLocation) support.getDropLocation();
                   int pos = dropStelle.getIndex();
                   
                   // Wenn Quelle und Ziel identisch sind, verschiebe nur, wenn
                   // die Einfgeposition auerhalb des zu verschiebenden
                   // Textes liegt
                   if((target == source) 
                      && (pos >= p0.getOffset()) 
                      && (pos <= p1.getOffset())) {
                      
                      shouldRemove = false;
                      return true;
                   }
                   
                   target.insert(str.toString(), pos);
                   
                } else {
                   // kein Drop, sondern Paste-Aktion
                   if (str != null)
                      target.replaceSelection(str);
                }
                                                
                return true;
            }
        } catch (UnsupportedFlavorException e) {
            System.console().printf("Drag-Daten werden nicht untersttzt\n");
        } catch (IOException e) {
            System.console().printf("I/O-Exception\n");
        }
        return false;
    }

   /*
     * erzeugt ein Transferable-Objekt mit den zu bertragenden Drag-Daten
     */
    protected Transferable createTransferable(JComponent c) {

        // Vorab prfen, ob sich hinter der Komponente c auch wirklich 
        // eine JTextArea-Komponente verbirgt 
        if (!(c instanceof JTextArea))
            return null;

        source = (JTextArea) c;

        int start = source.getSelectionStart();
        int end = source.getSelectionEnd();
        if (start == end) {
            return null;

        } else {
           try {
               // Anfangs- und Endposition des markierten Textes
               // im Dokument merken 
               Document doc = source.getDocument();
               p0 = doc.createPosition(start);
               p1 = doc.createPosition(end);
           } catch (BadLocationException e) {
               System.console().printf("Text kann nicht verschoben werden.\n");
           }

           shouldRemove = true;
           String data = source.getSelectedText();
           return new StringSelection(data);
        }
    }

    public int getSourceActions(JComponent c) {
        return COPY_OR_MOVE;
    }

   /*
     * Am Ende einer String-DROP-Aktion den Text in der Quelle lschen.
     */
    protected void exportDone(JComponent c, Transferable data, int action) {

        if (shouldRemove && (action == MOVE)) {
            if ((p0 != null) && (p1 != null) &&
                (p0.getOffset() != p1.getOffset())) {
                try {
                    JTextArea source = (JTextArea) c;
                    source.getDocument().remove(p0.getOffset(), 
                                             p1.getOffset() - p0.getOffset());
                } catch (BadLocationException e) {
                    System.console().printf("Text kann nicht gelscht werden.\n");
                }
            }
        }
        source = null;
    }

} 
